@@ -5,6 +5,8 @@
 
 # contributions by Milan Bouchet-Valat, Richard Heiberger, Duncan Murdoch, Erich Neuwirth, Brian Ripley
 
+# --------- MOD TRACER modifications by Jordi Cuadros
+
 Commander <- function(){
     library(Rcmdr, quietly=TRUE)
 
@@ -29,7 +31,13 @@ Commander <- function(){
         assignInMyNamespace(".RcmdrEnv", NULL)
         
     }
+
     RStudioP <- function() nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY")) # to detect RStudio
+    
+######### MOD TRACER    
+    RStudioP <- function() FALSE 
+######### END MOD TRACER
+    
     DESCRIPTION <- readLines(file.path(find.package("Rcmdr"), "DESCRIPTION")[1])
     RcmdrVersion <- trim.blanks(sub("^Version:", "",
         grep("^Version:", DESCRIPTION, value=TRUE)))
@@ -748,7 +756,14 @@ Commander <- function(){
     .commander <- CommanderWindow()
     tcl("wm", "iconphoto", .commander, "-default", "::image::RlogoIcon")
     tkwm.geometry(.commander, placement)
-    tkwm.title(.commander, gettextRcmdr("R Commander"))
+    
+######### MOD TRACER
+
+    # tkwm.title(.commander, gettextRcmdr("R Commander"))
+    tkwm.title(.commander, gettextRcmdr("R Commander TRACED"))
+
+######### END MOD TRACER
+    
     tkwm.protocol(.commander, "WM_DELETE_WINDOW", 
         if (getRcmdr("quit.R.on.close")) closeCommanderAndR else CloseCommander)
     topMenu <- tkmenu(.commander)
@@ -1145,6 +1160,11 @@ justDoIt <- function(command) {
     }
     checkWarnings(readLines(messages.connection))
     if (getRcmdr("RStudio")) Sys.sleep(0)
+    
+######### MOD TRACER
+    trace(command,result)
+######### END MOD TRACER    
+
     result
 }
 
@@ -1246,6 +1266,11 @@ doItAndPrint <- function(command, log=TRUE, rmd=log) {
         checkWarnings(readLines(messages.connection))
     }
     if (getRcmdr("RStudio")) Sys.sleep(0)
+    
+######### MOD TRACER
+    trace(command,result)
+######### END MOD TRACER    
+    
     result
 }
 
@@ -1385,3 +1410,66 @@ popCommand <- function(keep=FALSE){
     if (!keep) putRcmdr("commandStack", c(stack[-1], NA))
     lastCommand
 }
+
+
+######### MOD TRACER
+trace <- function (command, result, action="EXECUTE") 
+{
+  if (!exists("tr_user") || !exists("tr_session")) {
+    tr_user <<- paste("user_", floor(runif(1) * 1e+06), sep = "")
+    tr_session <<- format(Sys.time(), "%Y%m%d%H%M%OS3")
+    tr_number <<- 1
+  }
+  tr_xml <- "<EVENT application='Rcmdr-20170112' action='"
+  tr_xml <- paste(tr_xml, action, "' ", sep = "")
+  tr_xml <- paste(tr_xml, "user='", tr_user, "' ", sep = "")
+  tr_xml <- paste(tr_xml, "session='", tr_session, "' ", sep = "")
+  tr_xml <- paste(tr_xml, "number='", tr_number, "' ", sep = "")
+  tr_xml <- paste(tr_xml, "time='", format(Sys.time(), "%Y%m%d%H%M%OS3"), 
+                  "' ", sep = "")
+  tr_xml <- paste(tr_xml, "type='active'>", sep = "")
+  tr_xml <- paste(tr_xml, "<PARAM name='Command' value='", 
+                  URLencode(command, reserved=TRUE), "' />",sep = "")
+  tr_xml <- paste(tr_xml, "</EVENT>", sep = "")
+  write(tr_xml, file = paste("log_",format(Sys.time(), "%Y%m%d"),".txt",sep=""), append = TRUE)
+  tr_number <<- tr_number + 1
+  
+  if (substr(command, 1, 4) == "fix(") {
+    objectName<-substr(command, 5, nchar(command) - 1)
+    dataToLog <- paste(objectName, "colnames", paste(eval(parse(text=paste("colnames(",objectName,")",sep=""))),
+                                                     collapse="~~"),
+                       paste(eval(parse(text = objectName)),
+                             collapse = ";"), sep = ":")
+  } else if(substr(command, 1, 5) == "load(") {
+    objectName<-result
+    dataToLog <- paste(objectName, "colnames", paste(eval(parse(text=paste("colnames(",objectName,")",sep=""))),
+                                                     collapse="~~"),
+                       paste(eval(parse(text = objectName)),
+                             collapse = ";"), sep = ":")
+  } else {
+    suppressWarnings({
+      dataToLog <- ifelse(is.null(class(result)) | is.na(class(result)), "", 
+                        paste(class(result),"~~",sep=""))
+      dataToLog <- paste(dataToLog,
+                       ifelse(is.null(names(result)) | is.na(names(result)), "", 
+                        paste(paste(names(result),collapse=","),"~~",sep="")))
+      dataToLog <- paste(dataToLog,paste(print(result), collapse = "~~"),sep="")
+    })
+  }
+  
+  tr_xml <- "<EVENT application='Rcmdr-20170112' action='"
+  tr_xml <- paste(tr_xml, action, "' ", sep = "")
+  tr_xml <- paste(tr_xml, "user='", tr_user, "' ", sep = "")
+  tr_xml <- paste(tr_xml, "session='", tr_session, "' ", sep = "")
+  tr_xml <- paste(tr_xml, "number='", tr_number, "' ", sep = "")
+  tr_xml <- paste(tr_xml, "time='", format(Sys.time(), "%Y%m%d%H%M%OS3"), 
+                  "' ", sep = "")
+  tr_xml <- paste(tr_xml, "type='reactive'>", sep = "")
+  tr_xml <- paste(tr_xml, "<PARAM name='Result' value='", URLencode(dataToLog, reserved=TRUE), 
+                  "' />", sep = "")
+  tr_xml <- paste(tr_xml, "</EVENT>", sep = "")
+  write(tr_xml, file = paste("log_",format(Sys.time(), "%Y%m%d"),".txt",sep=""), append = TRUE)
+  tr_number <<- tr_number + 1
+}
+######### END MOD TRACER
+
